/*
 * i2c_dma.c
 *
 *  Created on: Sep 14, 2022
 *      Author: hussamaldean
 */

#include "i2c_dma.h"

#include "stm32f7xx.h"


volatile uint8_t rx_finished;
volatile uint8_t tx_finished;


/*
 * The channel is Channel 1
 * DMA1_Stream5 is I2C1_RX
 * DMA1_Stream6 is I2C1_TX
 *
 * */


void i2c_dma_init(uint32_t timing)
	{
		#define AF4 0x04
		#define CH1	0x01
		RCC->AHB1ENR|=RCC_AHB1ENR_GPIOBEN;
		GPIOB->MODER|=GPIO_MODER_MODER8_1|GPIO_MODER_MODER9_1;
		GPIOB->MODER&=~(GPIO_MODER_MODER8_0|GPIO_MODER_MODER9_0);
		GPIOB->OTYPER|=GPIO_OTYPER_OT8|GPIO_OTYPER_OT9;
		GPIOB->AFR[1]|=(AF4<<0)|(AF4<<4);

		RCC->APB1ENR|=RCC_APB1ENR_I2C1EN;

		I2C1->CR1 &=~I2C_CR1_PE;
		/*Enable I2C*/
		I2C1->TIMINGR=timing;
		I2C1->CR1 |=I2C_CR1_PE;

		/*Configuring DMA1_Stream5*/
		RCC->AHB1ENR|=RCC_AHB1ENR_DMA1EN;
		DMA1_Stream5->CR &=~DMA_SxCR_EN;
		while((DMA1_Stream5->CR & DMA_SxCR_EN)==1);
		DMA1_Stream5->CR|=(CH1<<25)|DMA_SxCR_MINC|DMA_SxCR_TCIE;
		NVIC_EnableIRQ(DMA1_Stream5_IRQn);
		DMA1_Stream5->PAR=(uint32_t )&I2C1->RXDR;

		/*Configuring DMA1_Stream6*/
		DMA1_Stream6->CR &=~DMA_SxCR_EN;
		while((DMA1_Stream6->CR & DMA_SxCR_EN)==1);
		DMA1_Stream6->CR|=(CH1<<25)|DMA_SxCR_MINC|DMA_SxCR_TCIE|DMA_SxCR_DIR_0;
		NVIC_EnableIRQ(DMA1_Stream6_IRQn);
		DMA1_Stream6->PAR=(uint32_t )&I2C1->TXDR;


	}


void i2c_write_dma(uint8_t slav_add, uint8_t *data, uint8_t length)
	{

			/*Enable I2C*/
			I2C1->CR1 |=I2C_CR1_PE;
			/*Enable DMA TX*/
			I2C1->CR1|=I2C_CR1_TXDMAEN;
			/*Set slave address*/
			I2C1->CR2=(slav_add<<1);
			/*7-bit addressing*/
			I2C1->CR2&=~I2C_CR2_ADD10;
			/*Set number to transfer to length for write operation*/
			I2C1->CR2|=(length<<I2C_CR2_NBYTES_Pos);
			/*Set the mode to write mode*/
			I2C1->CR2&=~I2C_CR2_RD_WRN;
			/*hardware end*/
			I2C1->CR2|=I2C_CR2_AUTOEND;

			DMA1_Stream6->M0AR=(uint32_t )data;
			DMA1_Stream6->NDTR=length;
			DMA1_Stream6->CR |=DMA_SxCR_EN;
			/*Generate start*/
			I2C1->CR2|=I2C_CR2_START;
			while(tx_finished==0);
			tx_finished=0;
	}


void i2c_read_dma(uint8_t slav_add, uint8_t memadd, uint8_t *data, uint8_t length)
	{

		/*Enable I2C*/
		I2C1->CR1 |=I2C_CR1_PE;
		/*Disable DMA for short period of time*/
		/*Set slave address*/
		I2C1->CR2=(slav_add<<1);
		/*7-bit addressing*/
		I2C1->CR2&=~I2C_CR2_ADD10;
		/*Set number to transfer to 1 for write operation*/
		I2C1->CR2|=(1<<I2C_CR2_NBYTES_Pos);
		/*Set the mode to write mode*/
		I2C1->CR2&=~I2C_CR2_RD_WRN;
		/*Software end*/
		I2C1->CR2&=~I2C_CR2_AUTOEND;
		/*Generate start*/
		I2C1->CR2|=I2C_CR2_START;
		/*Wait until transfer is completed*/
		while(!(I2C1->ISR & I2C_ISR_TC))
			{
				/*Check if TX buffer is empty*/
				if(I2C1->ISR & I2C_ISR_TXE)
					{
						/*send memory address*/
						I2C1->TXDR = (memadd);
					}
			}
	/*Reset I2C*/
	I2C1->CR1 &=~I2C_CR1_PE;
	I2C1->CR1 |=I2C_CR1_PE;
	/*Enable DMA TX*/
	I2C1->CR1|=I2C_CR1_RXDMAEN;
	/*Set slave address*/
	I2C1->CR2=(slav_add<<1);
	/*Set mode to read operation*/
	I2C1->CR2|=I2C_CR2_RD_WRN;
	/*Set length to the required length*/
	I2C1->CR2|=((length)<<I2C_CR2_NBYTES_Pos);
	/*auto generate stop after transfer completed*/
	I2C1->CR2|=I2C_CR2_AUTOEND;
	DMA1_Stream5->M0AR=(uint32_t)data;
	DMA1_Stream5->NDTR=length;
	DMA1_Stream5->CR |=DMA_SxCR_EN;
	/*Generate start*/
	I2C1->CR2|=I2C_CR2_START;
	while(rx_finished==0);
	rx_finished=0;


	}



void DMA1_Stream5_IRQHandler(void)
	{
		if(DMA1->HISR & DMA_HISR_TCIF5){
		rx_finished=1;
		DMA1->HIFCR=DMA_HIFCR_CTCIF5;
		}

	}



void DMA1_Stream6_IRQHandler(void)
	{
	if(DMA1->HISR & DMA_HISR_TCIF6){
		tx_finished=1;
		DMA1->HIFCR=DMA_HIFCR_CTCIF6;
		}

	}
